//
//  IODOrder.m
//  iOSDiner
//
//  Created by Adam Burkepile on 1/29/12.
//  Copyright (c) 2012 Adam Burkepile. All rights reserved.
//  Edited by James Prete on 2/3/2013.
//  Copyright (c) 2013 James Prete - your•bot. All rights reserved.
//  Edited by Chris LaPollo on 3/5/13
//  Copyright (c) 2012 Chris LaPollo. All rights reserved.


#import "IODOrder.h"
#import "IODItem.h"

@implementation IODOrder
@synthesize orderItems;
- (IODItem*)findKeyForOrderItem:(IODItem*)searchItem {
  // 1 - Find the matching item index
  NSIndexSet* indexes = [[orderItems allKeys] indexesOfObjectsPassingTest:^BOOL(IODItem* key, NSUInteger idx, BOOL *stop) {
    return [searchItem.name isEqualToString:key.name] && searchItem.price == key.price;
  }];
	// 2 - Return first matching item
  if ([indexes count] >= 1) {
    IODItem* key = [orderItems allKeys][[indexes firstIndex]];
    return key;
  }
	// 3 - If nothing is found
  return nil;
}

- (NSMutableDictionary *)orderItems
{
  if (!orderItems) {
    orderItems = [NSMutableDictionary new];
  }
  return orderItems;
}

- (NSString*)orderDescription
{
	// 1 - Create description string
  NSMutableString* orderDescription = [NSMutableString new];
	// 2 - Sort the order items by name
  NSArray* keys = [[orderItems allKeys] sortedArrayUsingComparator:^NSComparisonResult(IODItem* item1, IODItem* item2) {
    return [item1.name compare:item2.name];
  }];
	// 3 - Enumerate items and add item name and quantity to description
  [keys enumerateObjectsUsingBlock:^(IODItem* item, NSUInteger idx, BOOL *stop) {
    NSNumber* quantity = (NSNumber*)orderItems[item];
    [orderDescription appendFormat:@"%@ x%@\n", item.name, quantity];
  }];
	// 4 - Return order description
  return [orderDescription copy];
}

- (void)addItemToOrder:(IODItem*)inItem
{
	// 1 - Find item in order list
  IODItem* key = [self findKeyForOrderItem:inItem];
	// 2 - If the item doesn't exist, add it
  if (!key) {
    orderItems[inItem] = @1;
  } else {
		// 3 - If item exists, update the quantity
    NSNumber* quantity = (NSNumber*)orderItems[key];
    int intQuantity = [quantity intValue];
    intQuantity++;
		// 4 - Update order items list with new quantity
    orderItems[key] = (id<NSCopying>)[NSNumber numberWithInt:intQuantity];
  }
}

- (void)removeItemFromOrder:(IODItem*)inItem
{
	// 1 - Find the item in order list
  IODItem* key = [self findKeyForOrderItem:inItem];
	// 2 - We remove the item only if it exists
  if (key)
  {
		// 3 - Get the quanity and decrement by one
    NSNumber* quantity = orderItems[key];
    int intQuantity = [quantity intValue];
    intQuantity--;
		// 4 - Remove object from array
    if (intQuantity <= 0) {
      [orderItems removeObjectForKey:key];
    } else {
      // 5 - Add a new object with updated quantity only if quantity > 0
      //            if (intQuantity > 0)
      orderItems[key] = [NSNumber numberWithInt:intQuantity];
    }
  }
}

- (float)totalOrder
{
	// 1 - Define and initialize the total variable
  __block float total = 0.0;
	// 2 - Block for calculating total
  float (^itemTotal)(float,int) = ^float(float price, int quantity) {
    return price * quantity;
  };
	// 3 - Enumerate order items to get total
  [orderItems enumerateKeysAndObjectsUsingBlock:^(IODItem* item, NSNumber* quantity, BOOL *stop) {
    int intQuantity = [quantity intValue];
    total += itemTotal(item.price, intQuantity);
  }];
	// 4 - Return total
  return total;
}
@end